@@ -1,6 +1,6 @@
 /*
     Scan and print all the PC/SC devices available
-    Copyright (C) 2001-2009  Ludovic Rousseau <ludovic.rousseau@free.fr>
+    Copyright (C) 2001-2011  Ludovic Rousseau <ludovic.rousseau@free.fr>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
 
-/* $Id: pcsc_scan.c 5161 2010-08-18 21:36:11Z rousseau $ */
+/* $Id: pcsc_scan.c 5892 2011-08-19 09:32:46Z rousseau $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -87,7 +87,7 @@ int main(int argc, char *argv[])
 	int pnp = TRUE;
 
 	printf("PC/SC device scanner\n");
-	printf("V " VERSION " (c) 2001-2009, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
+	printf("V " VERSION " (c) 2001-2011, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
 	printf("Compiled with PC/SC lite version: " PCSCLITE_VERSION_NUMBER "\n");
 
 	while ((opt = getopt(argc, argv, "Vhn")) != EOF)
@@ -150,11 +150,17 @@ int main(int argc, char *argv[])
 	rgReaderStates[0].dwCurrentState = SCARD_STATE_UNAWARE;
 
 	rv = SCardGetStatusChange(hContext, 0, rgReaderStates, 1);
-	if (rgReaderStates[0].dwEventState && SCARD_STATE_UNKNOWN)
+	if (rgReaderStates[0].dwEventState & SCARD_STATE_UNKNOWN)
 	{
-		printf("PnP reader name not supported. Using polling.\n");
+		timeout = TIMEOUT;
+		printf("%sPlug'n play reader name not supported. Using polling every %ld ms.%s\n", magenta, timeout, color_end);
 		pnp = FALSE;
 	}
+	else
+	{
+		timeout = INFINITE;
+		printf("%sUsing reader plug'n play mechanism%s\n", magenta, color_end);
+	}
 
 get_readers:
 	/* free memory possibly allocated in a previous loop */
@@ -282,15 +288,13 @@ get_readers:
 	 * We only stop in case of an error
 	 */
 	if (pnp)
-	{
-		timeout = INFINITE;
 		nbReaders++;
-	}
-	else
-		timeout = TIMEOUT;
+
 	rv = SCardGetStatusChange(hContext, timeout, rgReaderStates_t, nbReaders);
 	while ((rv == SCARD_S_SUCCESS) || (rv == SCARD_E_TIMEOUT))
 	{
+		time_t t;
+
 		if (pnp)
 		{
 			if (rgReaderStates_t[nbReaders-1].dwEventState &
@@ -301,16 +305,18 @@ get_readers:
 		{
 			/* A new reader appeared? */
 			if ((SCardListReaders(hContext, NULL, NULL, &dwReaders)
-						== SCARD_S_SUCCESS) && (dwReaders != dwReadersOld))
+				== SCARD_S_SUCCESS) && (dwReaders != dwReadersOld))
 				goto get_readers;
 		}
 
+		/* Timestamp the event as we get notified */
+		t = time(NULL);
+		printf("\n%s", ctime(&t));
+
 		/* Now we have an event, check all the readers in the list to see what
 		 * happened */
 		for (current_reader=0; current_reader < nbReaders; current_reader++)
 		{
-			time_t t;
-
 			if (rgReaderStates_t[current_reader].dwEventState &
 				SCARD_STATE_CHANGED)
 			{
@@ -328,11 +334,8 @@ get_readers:
 			 * above.
 			 */
 
-			/* Timestamp the event as we get notified */
-			t = time(NULL);
-
 			/* Specify the current reader's number and name */
-			printf("\n%s Reader %d: %s%s%s\n", ctime(&t), current_reader,
+			printf("Reader %d: %s%s%s\n", current_reader,
 				magenta, rgReaderStates_t[current_reader].szReader,
 				color_end);
 
@@ -399,13 +402,15 @@ get_readers:
 				else
 					atr[0] = '\0';
 
-				printf("%s%s%s\n\n", magenta, atr, color_end);
+				printf("%s%s%s\n", magenta, atr, color_end);
 
 				/* force display */
 				fflush(stdout);
 
 				if (analyse_atr)
 				{
+					printf("\n");
+
 					sprintf(atr_command, ATR_PARSER " '%s'", atr);
 					if (system(atr_command))
 						perror(atr_command);
@@ -419,7 +424,7 @@ get_readers:
 
 	/* If we get out the loop, GetStatusChange() was unsuccessful */
 	test_rv("SCardGetStatusChange", rv, hContext);
-	
+
 	/* We try to leave things as clean as possible */
 	rv = SCardReleaseContext(hContext);
 	test_rv("SCardReleaseContext", rv, hContext);
